/*

  ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒
                      1. Tocador de pulso com escala cromática
                             Com o Módulo BotPotLED Gl
                             Por Glauber Santiago - 2019
  ▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒▒

  ▒▒ Descrição e funcionamento
  Este sketch utiliza o Módulo BotPotLED Gl, com 2 botões, um potenciômetro e 3 LEDs 
  (vermelho, amarelo e verde). Adicionalmente utiliza-se um autofalante.
  Trata-se de um instrumento musical. As notas escolhidas pelo manejo do potenciômetro 
  são emitidas em pulsos de duração variável, conforme o controle do usuário.
  Ao ser ligado o Arduino é emitida uma vinheta de abertura visual e sonora. Ao final 
  da vinheta é emitido o intervalo melódico de primeira justa. Indicando 1, que representa 
  o presente   sketch elaborado para o Módulo BotPotLED Gl.
  Após a abertura o sistema fica em modo "pausa", com os LEDs oscilando sua luminosidade 
  e sem nenhum som sendo emitido.
  Ao se pressionar o botão 1 (o da esquerda) o modo "pausa" é interrompido. Então conforme 
  a posição do potenciômetro é emitida uma nota da escala cromática temperada em um pulso constante.
  A duração do pulso é modificada com os botões. Botão 1 aumenta da duração do pulso (fica 
  mais lento) e o Botão 2 diminui a sua duração (ficando mais rápido).
  Para se retornar ao modo "pause" é necessário o acionamento simultâneo dos botões 1 e 2.

  Resumindo os recursos:
  LED vermelho -> No modo pausa fica piscando de forma intermitente. Fora do modo pausa indica o acionamento do botão 1.
  LED amarelo -> No modo pausa fica piscando de forma intermitente. Fora do modo pausa indica o início de cada pulso.
  LED verde -> No modo pausa fica piscando de forma intermitente.
  Potenciômetro -> Varia a nota musical emitida.
  Botão 1 -> Aumenta a duração do pulso ou sai do modo "pausa".
  Botão 2 -> Diminui a duração do pulso ou sai do modo "pausa".

  Ideias para uso didático:
  1-Duelo de músicos em uníssono: Um usuário seleciona uma nota para ser tocada, 
  então outro deve buscar tocar a mesma nota em outro Módulo (ou em outro instrumento musical).
  2-Duelo de músicos em terça maior: Um usuário seleciona uma nota para ser tocada, 
  então outro deve buscar tocar a nota uma terça maior acima em outro Módulo.
  3-Uso como instrumento musical em circunstância de apreciação musical relativas à 
  frequência, andamento, duração e timbres sintetizados.

   Ideias de variação na programação:
  -Modificar as variáveis "notaMaisGrave" e "notaMaisAguda".
  -Modificar a quantidade de notas diferentes emitidas por pulso, oitavando, por exemplo.
  -Executar uma tríade arpejada no lugar de uma nota apenas.
  -Criar motivo rítmico para ser executado em cada pulso.
  -Mudar melodia da vinheta de abertura.

  ▒▒ Circuito:
  Utilizando-se um Arduino UNO como modelo.
  Externo:
  1 autofalante, de 4 ou 8 ohms, intermediado em um dos terminais por um resistor de 330 ohms (este resistor pode ser opcional se o
  autofalante tiver mais de 8 ohms). Um dos terminais ligados no GND e o outro no pino 10, do Arduino.
  1 Arduino UNO.

  ▒▒ Módulo BotPotLED Gl:
  3 LEDs:
  LED vermelho: Positivo, intermediado por um resistor de 330 ohms, no pino 6;
  LED amarelo: Positivo, intermediado por um resistor de 330 ohms, no pino 5;
  LED verde: Positivo, intermediado por um resistor de 330 ohms, no pino 3;

  1 potenciômetro. Conector do centro no pino analógico (A0) do Arduino. Os pinos laterais do potenciômetro
  vão para o GND e o 5v do Arduino.

  2 pushbutton entre cada pino, conforme a indicação a seguir, e o GND. Ou seja, um terminal do pushbutton no GND e o outro em pino. Conforme segue:
  Botão 1 no pino 10
  Botão 2 no pino 9

  No módulo os cabos externos para conexão com o arduino estão dispostos de forma alinhada com as partes: LEDs (3 cabos), Potenciômetro (2 cabos)
  e Botões (2 cabos). Além dos respectivos conjuntos de cabos existe um cabo de terra (GND) de cor preta saindo do módulo.


  Resumo da pinagem no arduino:
  0:
  1:
  2:
  3:Positivo do LED verde intermediado por resistor. (Cabo verde dos LEDs do Módulo BotPotLED Gl)
  4:
  5:Positivo do LED amarelo intermediado por resistor. (Cabo amarelo dos LEDs do Módulo BotPotLED Gl)
  6:Positivo do LED vermelho intermediado por resistor. (Cabo vermelho dos LEDs do Módulo BotPotLED Gl)
  7:
  8:Botão 2. (Cabo amarelo dos Botões do Módulo BotPotLED Gl)
  9:Botão 1. (Cabo azul dos Botões do Módulo BotPotLED Gl)
  10: Terminal do autofalante intermediado por resistor. O Resistor é dispensável se autofalante tiver mais que 8 ohms.
  11: 
  12:
  13:
  GND: Cabo negativo do autofalante.(Cabo preto do Módulo BotPotLED Gl)
  AREF:
  A5:
  A4:
  A3:
  A2:
  A1:
  A0: Contato central do potenciômetro. (Cabo azul do potenciômetro do Módulo BotPotLED Gl)
  Vin:
  GND:
  GND:Cabo Preto dos vários sinais negativos. (Cabo preto do Módulo BotPotLED Gl)
  5V: Positivo (um dos terminais laterais) do potenciômetro. (Cabo vermelho do potenciômetro do Módulo BotPotLED Gl)
  3.3V:
  Reset:
  IOREF:


  ▒▒ Software:


  Além disso foi utilizado o presente sketch a ser carregado no micro controlador.

  Fórmula para calcular frequência de uma nota no sistema temperado:
  440 * pow(2.0, (notaDesejada - 57) / 12.0)
  sendo: 96 = dó6; 84 = dó5; 72 = dó4; 60 = dó3 (central); 48 = dó2; 36 = dó1; etc

  Indicações de durações para o andamento 120 bpm
  semibreve = 2000;
  menina = 1000;
  semínima = 500;
  colcheia = 250;
  semicolcheia = 125;

*/

//Variáveis para a definição das portas do Arduino
int pinoDoAutofalante = 10;
int pinoDoBotao1 = 9;
int pinoDoBotao2 = 8;
int pinoDoLEDVermelho = 6;
int pinoDoLEDAmarelo = 5;
int pinoDoLEDVerde = 3;
int pinoDoPotenciometro = A0;

//Variáveis para a definição inicial dos estados dos botões. HIGH significa desligado.
int estadoDoBotao1 = HIGH;
int estadoDoBotao2 = HIGH;

//Variáveis para o valor do potenciômetro
int valorDoPotenciometro;

//Variáveis para o resultado já mapeado de cada potenciômetro
int resultado;

//Demais variáveis para a programação
float mandaParaSerialPot;
int duracaoDoPulso = 100;

int pausa = 1;




//Definição da extensão, sendo: 96 = dó6; 84 = dó5; 72 = dó4; 60 = dó3 (central); 48 = dó2; 36 = dó1; etc
int notaMaisGrave = 12;
int notaMaisAguda = 96;

//A seguir temos o void Setup que é uma parte do programa que é executada inicialmente e uma única vez.
void setup()
{
  //Definição sobre os pinos do arduino
  pinMode(pinoDoBotao1, INPUT_PULLUP);//O modo INPUT_PULLUP faz com que seja dispensado o uso de um resistor no botão, porém faz com que ele inverta a situação de ligação. HIGH fica desligado e LOW fica ligado.
  pinMode(pinoDoBotao2, INPUT_PULLUP);
  pinMode(pinoDoLEDVermelho, OUTPUT);
  pinMode(pinoDoLEDAmarelo, OUTPUT);
  pinMode(pinoDoLEDVerde, OUTPUT);
  pinMode(pinoDoAutofalante, OUTPUT);
  pinMode(pinoDoPotenciometro, INPUT);
  //Inicia o monitor serial
  Serial.begin(9600);
  //Executa comendos de apresentação do Sketch (1)
  vinhetaAbertura();




}

// A seguir temos o void loop que fica repetindo para sempre os comandos em seu interior.
void loop()
{

  lacoDePausa();

  //informa que os pino dos LEDs devem estar no estado LOW, que, neste caso é "desligado".
  nenhumLED ();

  verificacaoDeApertoDoBotao1();//Chama uma vez a função com este nome que está após o void loop.
  verificacaoDeApertoDoBotao2();//Chama uma vez a função com este nome que está após o void loop.
  verificaPotenciometro();//Chama uma vez a função com este nome que está após o void loop.

  delay (duracaoDoPulso * 0.5);//Faz com que o programa fique parado em seu fluxo conforme a duração indicada.
  verificacaoDeApertoDoBotao1();//Chama uma vez a função com este nome que está após o void loop.
  verificacaoDeApertoDoBotao2();//Chama uma vez a função com este nome que está após o void loop.
  verificaPotenciometro();//Chama uma vez a função com este nome que está após o void loop.

  tone(pinoDoAutofalante, 440 * pow(2.0, (resultado - 57) / 12.0), duracaoDoPulso - 20);//Emite um som pelo pinoDoAutofalante. Parâmetros (pino, frequência em hertz, duração em milissegundos)
  //Serial.println (440 * pow(2.0, (resultado - 57) / 12.0));
  //tone(pinoDoAutofalante, resultado, duracaoDoPulso - 20);//Emite um som pelo pinoDoAutofalante. Parâmetros (pino, frequência em hertz, duração em milissegundos)
  digitalWrite (pinoDoLEDAmarelo, HIGH);//informa que o pino amarelo deve estar no estado HIGH, que, neste caso é "ligado".
  delay (duracaoDoPulso * 0.5);

  verificacaoDePausa ();

}
